// Copyright 2018 Google Inc. All Rights Reserved.
//
// Use of this source code is governed by an MIT-style
// license that can be found in the LICENSE file or at
// https://opensource.org/licenses/MIT.

#include "brotli.h"

#include <string.h>  // memcpy
#include <memory>
#include <brotli/decode.h>
#include <brotli/encode.h>
#include "status.h"

namespace pik {

/* BrotliCompress(int, const uint8_t* ...) removed */

/* BrotliCompress(int, const PaddedBytes& ...) removed */

Status BrotliDecompress(const uint8_t* in, size_t max_input_size,
                        size_t max_output_size, size_t* PIK_RESTRICT bytes_read,
                        PaddedBytes* PIK_RESTRICT out) {
  std::unique_ptr<BrotliDecoderState, decltype(BrotliDecoderDestroyInstance)*>
      s(BrotliDecoderCreateInstance(nullptr, nullptr, nullptr),
        BrotliDecoderDestroyInstance);
  if (!s) return PIK_FAILURE("BrotliDecoderCreateInstance failed");

  const size_t kBufferSize = 128 * 1024;
  PaddedBytes temp_buffer(kBufferSize);

  size_t avail_in = max_input_size;
  if (max_input_size == 0) return false;
  const uint8_t* next_in = in;
  BrotliDecoderResult code;

  while (1) {
    size_t out_size;
    size_t avail_out = kBufferSize;
    uint8_t* next_out = temp_buffer.data();
    code = BrotliDecoderDecompressStream(s.get(), &avail_in, &next_in,
                                         &avail_out, &next_out, nullptr);
    out_size = next_out - temp_buffer.data();
    out->resize(out->size() + out_size);
    if (out->size() > max_output_size)
      return PIK_FAILURE("Brotli output too large");
    memcpy(out->data() + out->size() - out_size, temp_buffer.data(), out_size);
    if (code != BROTLI_DECODER_RESULT_NEEDS_MORE_OUTPUT) break;
  }
  if (code != BROTLI_DECODER_RESULT_SUCCESS)
    return PIK_FAILURE("Brotli decompression failed");
  *bytes_read += (max_input_size - avail_in);
  return true;
}

Status BrotliDecompress(const PaddedBytes& in, size_t max_output_size,
                        size_t* PIK_RESTRICT bytes_read,
                        PaddedBytes* PIK_RESTRICT out) {
  return BrotliDecompress(in.data(), in.size(), max_output_size, bytes_read,
                          out);
}

}  // namespace pik
