// Copyright 2017 Google Inc. All Rights Reserved.
//
// Use of this source code is governed by an MIT-style
// license that can be found in the LICENSE file or at
// https://opensource.org/licenses/MIT.

#ifndef PIK_COMPRESSED_IMAGE_H_
#define PIK_COMPRESSED_IMAGE_H_

#include <stddef.h>
#include <stdint.h>

#include "adaptive_reconstruction.h"
#include "bit_reader.h"
#include "block_dictionary.h"
#include "color_correlation.h"
#include "common.h"
#include "compressed_image_fwd.h"
#include "data_parallel.h"
#include "headers.h"
#include "image.h"
#include "multipass_handler.h"
#include "noise.h"
#include "padded_bytes.h"
#include "pik_info.h"
#include "pik_params.h"
#include "quantizer.h"

// Methods to encode (decode) an image into (from) the bit stream:
// initialization of per-pass information and per-group information, actual
// computation of quantized coefficients, and encoding, plus corresponding
// methods for the decoder.

namespace pik {

struct GradientMap;

/* InitializeFrameEncCache() removed */

/* InitializeEncCache() removed */

/* ComputeCoefficients() removed */

/* EncodeToBitstream() removed */

// Decodes AC coefficients from the bit stream, populating the AC
// fields of the decoder cache, and the corresponding rectangles in the global
// information (quant_field and ac_strategy) in the per-pass decoder cache.
template <bool first>
bool DecodeFromBitstream(const FrameHeader& frame_header,
                         const GroupHeader& header,
                         const PaddedBytes& compressed, BitReader* reader,
                         const Rect& group_rect, MultipassHandler* handler,
                         const size_t xsize_blocks, const size_t ysize_blocks,
                         const ColorCorrelationMap& cmap, const Rect& cmap_rect,
                         NoiseParams* noise_params, const Quantizer& quantizer,
                         FrameDecCache* PIK_RESTRICT frame_dec_cache,
                         GroupDecCache* PIK_RESTRICT group_dec_cache,
                         PikInfo* aux_out);

/* DequantImageAC() removed */

// Applies predictions to de-quantized AC coefficients, copies DC coefficients
// into AC, and does IDCT. Writes opsin IDCT values into `idct:idct_rect`.
void ReconOpsinImage(const FrameHeader& frame_header, const GroupHeader& header,
                     const Quantizer& quantizer, const Rect& block_group_rect,
                     FrameDecCache* PIK_RESTRICT frame_dec_cache,
                     GroupDecCache* PIK_RESTRICT group_dec_cache,
                     Image3F* PIK_RESTRICT idct, const Rect& idct_rect,
                     PikInfo* aux_out = nullptr, size_t downsample = 1);

// Finalizes the decoding of a pass by running per-pass post processing:
// smoothing and adaptive reconstruction. Writes linear sRGB to `idct` and
// shrinks it to `x/ysize` to undo prior padding.
// TODO(janwas): move NoiseParams into FrameHeader.
Status FinalizeFrameDecoding(Image3F* PIK_RESTRICT idct, size_t xsize,
                             size_t ysize, const FrameHeader& frame_header,
                             const NoiseParams& noise_params,
                             const Quantizer& quantizer,
                             const BlockDictionary& block_dictionary,
                             ThreadPool* pool, FrameDecCache* frame_dec_cache,
                             PikInfo* aux_out = nullptr, size_t downsample = 1);

}  // namespace pik

#endif  // PIK_COMPRESSED_IMAGE_H_
