// Copyright 2019 Google LLC
//
// Use of this source code is governed by an MIT-style
// license that can be found in the LICENSE file or at
// https://opensource.org/licenses/MIT.

#ifndef PIK_DCT_UTIL_H_
#define PIK_DCT_UTIL_H_

#include "common.h"
#include "data_parallel.h"
#include "image.h"

namespace pik {

/* ScatterBlock() removed */

template <size_t SX, size_t SY, typename T>
void GatherBlock(const T* PIK_RESTRICT row, size_t stride,
                 T* PIK_RESTRICT block, size_t block_stride) {
  constexpr size_t xblocks = SX / kBlockDim;
  constexpr size_t yblocks = SY / kBlockDim;
  for (size_t y = 0; y < SY; y++) {
    const T* PIK_RESTRICT current_row =
        row + (y & (yblocks - 1)) * stride + (y / yblocks) * kBlockDim;
    for (size_t x = 0; x < SX; x++) {
      size_t block_pos = y * SX + x;
      size_t block_row = block_pos / (xblocks * kDCTBlockSize);
      size_t block_idx = block_pos & (xblocks * kDCTBlockSize - 1);
      block[block_row * block_stride + block_idx] =
          current_row[(x & (xblocks - 1)) * kDCTBlockSize + (x / xblocks)];
    }
  }
}

// Fills a preallocated (N*N)*W x H `dct` with (N*N)x1 blocks produced by
// ComputeTransposedScaledDCT() from the corresponding NxN block of
// `image`. Note that `dct` coefficients are scaled by 1 / (N*N), so that
// ComputeTransposedScaledIDCT applied to each block or TransposedScaledIDCT
// will return the original input.
// REQUIRES: image.xsize() == N*W, image.ysize() == N*H
SIMD_ATTR void TransposedScaledDCT(const Image3F& image,
                                   Image3F* PIK_RESTRICT dct);

// Fills a preallocated N*W x N*H `idct` with NxN blocks produced by
// ComputeTransposedScaledIDCT() from the (N*N)x1 blocks of `dct`.
// REQUIRES: dct.xsize() == N*N*W, dct.ysize() == H
SIMD_ATTR void TransposedScaledIDCT(const Image3F& dct,
                                    Image3F* PIK_RESTRICT idct);

/* DCImage() removed */

/* FillDC() removed */

}  // namespace pik

#endif  // PIK_DCT_UTIL_H_
