// Copyright 2017 Google Inc. All Rights Reserved.
//
// Use of this source code is governed by an MIT-style
// license that can be found in the LICENSE file or at
// https://opensource.org/licenses/MIT.

#ifndef PIK_GAUSS_BLUR_H_
#define PIK_GAUSS_BLUR_H_

#include <stddef.h>
#include <vector>

#include "image.h"

namespace pik {

template <typename T>
std::vector<T> GaussianKernel(int radius, T sigma) {
  PIK_ASSERT(sigma > 0.0);
  std::vector<T> kernel(2 * radius + 1);
  const T scaler = -1.0 / (2 * sigma * sigma);
  double sum = 0.0;
  for (int i = -radius; i <= radius; ++i) {
    const T val = std::exp(scaler * i * i);
    kernel[i + radius] = val;
    sum += val;
  }
  for (int i = 0; i < kernel.size(); ++i) {
    kernel[i] /= sum;
  }
  return kernel;
}

/* Convolve(const ImageF& ...) removed */
/* Convolve(const Image3F& ...) removed */

/* Convolve(const ImageF& ...) removed */
/* Convolve(const Image3F& ...) removed */

/* ConvolveAndSample() removed */
/* ConvolveAndSample() removed */

/* ConvolveXSampleAndTranspose(const ImageF& ...) removed */
/* ConvolveXSampleAndTranspose(const Image3F& ...) removed */

}  // namespace pik

#endif  // PIK_GAUSS_BLUR_H_
