// Copyright 2017 Google Inc. All Rights Reserved.
//
// Use of this source code is governed by an MIT-style
// license that can be found in the LICENSE file or at
// https://opensource.org/licenses/MIT.

#include "quantizer.h"

#include <stdio.h>
#include <algorithm>
#include <sstream>
#include <vector>
#include "ac_strategy.h"

#include "arch_specific.h"
#include "common.h"
#include "compiler_specific.h"
#include "quant_weights.h"
#include "simd/simd.h"

namespace pik {

static const int kDefaultQuant = 64;

Quantizer::Quantizer(const DequantMatrices* dequant, int quant_xsize,
                     int quant_ysize)
    : Quantizer(dequant, quant_xsize, quant_ysize, kDefaultQuant,
                kGlobalScaleDenom / kDefaultQuant) {}

Quantizer::Quantizer(const DequantMatrices* dequant, int quant_xsize,
                     int quant_ysize, int quant_dc, int global_scale)
    : quant_xsize_(quant_xsize),
      quant_ysize_(quant_ysize),
      global_scale_(global_scale),
      quant_dc_(quant_dc),
      quant_img_ac_(quant_xsize_, quant_ysize_),
      dequant_(dequant) {
  RecomputeFromGlobalScale();

  FillImage(kDefaultQuant, &quant_img_ac_);

  memcpy(zero_bias_, kZeroBiasDefault, sizeof(kZeroBiasDefault));
}

// TODO(veluca): reclaim the unused bit in global_scale encoding.
std::string Quantizer::Encode(PikImageSizeInfo* info) const {
  std::stringstream ss;
  int global_scale = global_scale_ - 1;
  ss << std::string(1, global_scale >> 8);
  ss << std::string(1, global_scale & 0xff);
  ss << std::string(1, quant_dc_ - 1);
  if (info) {
    info->total_size += 3;
  }
  return ss.str();
}

bool Quantizer::Decode(BitReader* br) {
  int global_scale = br->ReadBits(8) << 8;
  global_scale |= br->ReadBits(8);
  global_scale_ = (global_scale & 0x7FFF) + 1;
  quant_dc_ = br->ReadBits(8) + 1;
  RecomputeFromGlobalScale();
  inv_quant_dc_ = inv_global_scale_ / quant_dc_;
  return true;
}

/* Quantizer::DumpQuantizationMap() removed */

/* QuantizeCoeffsDC() removed */

/* QuantizeRoundtripDC() removed */

}  // namespace pik
